@@ -157,8 +157,10 @@ connection_vpn_state_changed (NMVPNConnection *connection,
 NMVPNConnection *
 nm_vpn_manager_activate_connection (NMVPNManager *manager,
                                     NMConnection *connection,
+                                    NMActRequest *act_request,
                                     NMDevice *device,
-                                    const char *specific_object,
+                                    gboolean user_requested,
+                                    gulong user_uid,
                                     GError **error)
 {
 	NMSettingVPN *vpn_setting;
@@ -168,6 +170,7 @@ nm_vpn_manager_activate_connection (NMVPNManager *manager,
 
 	g_return_val_if_fail (NM_IS_VPN_MANAGER (manager), NULL);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
+	g_return_val_if_fail (NM_IS_ACT_REQUEST (act_request), NULL);
 	g_return_val_if_fail (NM_IS_DEVICE (device), NULL);
 	g_return_val_if_fail (error != NULL, NULL);
 	g_return_val_if_fail (*error == NULL, NULL);
@@ -204,7 +207,7 @@ nm_vpn_manager_activate_connection (NMVPNManager *manager,
 		return NULL;
 	}
 
-	vpn = nm_vpn_service_activate (service, connection, device, specific_object, error);
+	vpn = nm_vpn_service_activate (service, connection, act_request, device, user_requested, user_uid, error);
 	if (vpn) {
 		g_signal_connect (vpn, "vpn-state-changed",
 		                  G_CALLBACK (connection_vpn_state_changed),
@@ -368,7 +371,6 @@ try_add_service (NMVPNManager *self, const char *namefile)
 	service = nm_vpn_service_new (namefile, &error);
 	if (!service) {
 		nm_log_warn (LOGD_VPN, "failed to load VPN service file %s: (%d) %s",
-		             namefile,
 		             error ? error->code : -1,
 		             error && error->message ? error->message : "(unknown)");
 		g_clear_error (&error);
@@ -377,7 +379,7 @@ try_add_service (NMVPNManager *self, const char *namefile)
 
 	service_name = nm_vpn_service_get_dbus_service (service);
 	g_hash_table_insert (priv->services, (char *) service_name, service);
-	nm_log_info (LOGD_VPN, "VPN: loaded %s", service_name);
+	nm_log_info (LOGD_VPN, "VPN: loaded %s", service_name, service);
 }
 
 static void
@@ -409,7 +411,7 @@ vpn_dir_changed (GFileMonitor *monitor,
 			/* Stop active VPN connections and destroy the service */
 			nm_vpn_service_connections_stop (service, TRUE,
 			                                 NM_VPN_CONNECTION_STATE_REASON_SERVICE_STOPPED);
-			nm_log_info (LOGD_VPN, "VPN: unloaded %s", service_name);
+			nm_log_info (LOGD_VPN, "VPN: unloaded %s", service_name, service);
 			g_hash_table_remove (priv->services, service_name);
 		}
 		break;
